@@ -49,12 +49,21 @@ static GtkButton* new_action_button(const gchar* icon, const gchar* text)
 	return button;
 }
 
+Menu::Command Menu::m_settings_command = "xfce4-settings-manager";
+Menu::Command Menu::m_lockscreen_command = "xflock4";
+Menu::Command Menu::m_logout_command = "xfce4-session-logout";
+bool Menu::m_display_recent = false;
+bool Menu::m_position_search_alternate = false;
+bool Menu::m_position_commands_alternate = false;
+
 //-----------------------------------------------------------------------------
 
 Menu::Menu(XfceRc* settings) :
 	m_window(NULL),
 	m_layout_left(true),
 	m_layout_bottom(true),
+	m_layout_search_alternate(false),
+	m_layout_commands_alternate(false),
 	m_modified(false)
 {
 	m_geometry.x = 0;
@@ -134,22 +143,42 @@ Menu::Menu(XfceRc* settings) :
 	// Create search results
 	m_search_results = new SearchPage(this);
 
+	// Handle default page
+	if (!m_display_recent)
+	{
+		m_default_button = m_favorites_button;
+		m_default_page = m_favorites;
+	}
+	else
+	{
+		m_default_button = m_recent_button;
+		m_default_page = m_recent;
+	}
+
 	// Create box for packing children
 	m_vbox = GTK_BOX(gtk_vbox_new(false, 6));
 	gtk_container_add(GTK_CONTAINER(frame), GTK_WIDGET(m_vbox));
 	gtk_container_set_border_width(GTK_CONTAINER(m_vbox), 2);
 
-	// Create box for packing username and action buttons
+	// Create box for packing commands
+	m_commands_align = GTK_ALIGNMENT(gtk_alignment_new(1, 0, 0, 0));
+	m_commands_box = GTK_BOX(gtk_hbox_new(false, 0));
+	gtk_box_pack_start(m_commands_box, GTK_WIDGET(m_settings_button), false, false, 0);
+	gtk_box_pack_start(m_commands_box, GTK_WIDGET(m_lock_screen_button), false, false, 0);
+	gtk_box_pack_start(m_commands_box, GTK_WIDGET(m_log_out_button), false, false, 0);
+	gtk_container_add(GTK_CONTAINER(m_commands_align), GTK_WIDGET(m_commands_box));
+
+	// Create box for packing username, commands, and resize widget
 	m_title_box = GTK_BOX(gtk_hbox_new(false, 0));
 	gtk_box_pack_start(m_vbox, GTK_WIDGET(m_title_box), false, false, 0);
 	gtk_box_pack_start(m_title_box, GTK_WIDGET(m_username), true, true, 0);
-	gtk_box_pack_start(m_title_box, GTK_WIDGET(m_settings_button), false, false, 0);
-	gtk_box_pack_start(m_title_box, GTK_WIDGET(m_lock_screen_button), false, false, 0);
-	gtk_box_pack_start(m_title_box, GTK_WIDGET(m_log_out_button), false, false, 0);
+	gtk_box_pack_start(m_title_box, GTK_WIDGET(m_commands_align), false, false, 0);
 	gtk_box_pack_start(m_title_box, GTK_WIDGET(m_resizer->get_widget()), false, false, 0);
 
 	// Add search to layout
-	gtk_box_pack_start(m_vbox, GTK_WIDGET(m_search_entry), false, true, 0);
+	m_search_box = GTK_BOX(gtk_hbox_new(false, 6));
+	gtk_box_pack_start(m_vbox, GTK_WIDGET(m_search_box), false, true, 0);
+	gtk_box_pack_start(m_search_box, GTK_WIDGET(m_search_entry), true, true, 0);
 
 	// Create box for packing launcher pages and sidebar
 	m_contents_box = GTK_BOX(gtk_hbox_new(false, 6));
@@ -180,15 +209,20 @@ Menu::Menu(XfceRc* settings) :
 	gtk_container_add(GTK_CONTAINER(m_sidebar), viewport);
 	gtk_container_add(GTK_CONTAINER(viewport), GTK_WIDGET(m_sidebar_box));
 
+	GtkSizeGroup* sidebar_size_group = gtk_size_group_new(GTK_SIZE_GROUP_HORIZONTAL);
+	gtk_size_group_add_widget(sidebar_size_group, GTK_WIDGET(m_sidebar));
+	gtk_size_group_add_widget(sidebar_size_group, GTK_WIDGET(m_commands_align));
+
 	// Populate app menu
 	m_applications->load_applications();
 
 	// Show widgets
 	gtk_widget_show_all(GTK_WIDGET(m_vbox));
+	gtk_widget_hide(m_favorites->get_widget());
 	gtk_widget_hide(m_recent->get_widget());
 	gtk_widget_hide(m_applications->get_widget());
 	gtk_widget_hide(m_search_results->get_widget());
-	m_favorites_button->set_active(true);
+	m_default_button->set_active(true);
 	gtk_widget_show(frame);
 
 	// Resize to last known size
@@ -224,11 +258,23 @@ void Menu::hide()
 	// Hide window
 	gtk_widget_hide(GTK_WIDGET(m_window));
 
-	// Reset mouse cursor by forcing favorites to hide
-	gtk_widget_hide(m_favorites->get_widget());
+	// Update default page
+	if (m_display_recent && (m_default_page == m_favorites))
+	{
+		m_default_button = m_recent_button;
+		m_default_page = m_recent;
+	}
+	else if (!m_display_recent && (m_default_page == m_recent))
+	{
+		m_default_button = m_favorites_button;
+		m_default_page = m_favorites;
+	}
 
-	// Switch back to favorites
-	show_favorites();
+	// Reset mouse cursor by forcing default page to hide
+	gtk_widget_hide(m_default_page->get_widget());
+
+	// Switch back to default page
+	show_default_page();
 }
 
 //-----------------------------------------------------------------------------
@@ -244,11 +290,16 @@ void Menu::show(GtkWidget* parent, bool horizontal)
 	m_recent->get_view()->reload_icon_size();
 	m_applications->get_view()->reload_icon_size();
 
+	// Make sure commands are valid
+	check_command(m_settings_command, GTK_WIDGET(m_settings_button));
+	check_command(m_lockscreen_command, GTK_WIDGET(m_lock_screen_button));
+	check_command(m_logout_command, GTK_WIDGET(m_log_out_button));
+
 	// Make sure applications list is current; does nothing unless list has changed
 	m_applications->load_applications();
 
-	// Reset mouse cursor by forcing favorites to hide
-	gtk_widget_show(m_favorites->get_widget());
+	// Reset mouse cursor by forcing default page to hide
+	gtk_widget_show(m_default_page->get_widget());
 
 	GdkScreen* screen = NULL;
 	int parent_x = 0, parent_y = 0, parent_w = 0, parent_h = 0;
@@ -369,18 +420,76 @@ void Menu::show(GtkWidget* parent, bool horizontal)
 	{
 		layout_left = !layout_left;
 	}
+	if (m_layout_commands_alternate != m_position_commands_alternate)
+	{
+		m_layout_left = !layout_left;
+		m_layout_commands_alternate = m_position_commands_alternate;
+		if (m_layout_commands_alternate)
+		{
+			g_object_ref(m_commands_align);
+			gtk_container_remove(GTK_CONTAINER(m_title_box), GTK_WIDGET(m_commands_align));
+			gtk_box_pack_start(m_search_box, GTK_WIDGET(m_commands_align), false, false, 0);
+			g_object_unref(m_commands_align);
+		}
+		else
+		{
+			g_object_ref(m_commands_align);
+			gtk_container_remove(GTK_CONTAINER(m_search_box), GTK_WIDGET(m_commands_align));
+			gtk_box_pack_start(m_title_box, GTK_WIDGET(m_commands_align), false, false, 0);
+			g_object_unref(m_commands_align);
+		}
+	}
 	if (layout_left != m_layout_left)
 	{
 		m_layout_left = layout_left;
-		if (m_layout_left)
+		if (m_layout_left && m_layout_commands_alternate)
 		{
 			gtk_misc_set_alignment(GTK_MISC(m_username), 0.0f, 0.5f);
 
+			gtk_alignment_set(m_commands_align, 1, 0, 0, 0);
+			gtk_box_reorder_child(m_commands_box, GTK_WIDGET(m_settings_button), 0);
+			gtk_box_reorder_child(m_commands_box, GTK_WIDGET(m_lock_screen_button), 1);
+			gtk_box_reorder_child(m_commands_box, GTK_WIDGET(m_log_out_button), 2);
+
 			gtk_box_reorder_child(m_title_box, GTK_WIDGET(m_username), 0);
-			gtk_box_reorder_child(m_title_box, GTK_WIDGET(m_settings_button), 1);
-			gtk_box_reorder_child(m_title_box, GTK_WIDGET(m_lock_screen_button), 2);
-			gtk_box_reorder_child(m_title_box, GTK_WIDGET(m_log_out_button), 3);
-			gtk_box_reorder_child(m_title_box, GTK_WIDGET(m_resizer->get_widget()), 4);
+			gtk_box_reorder_child(m_title_box, GTK_WIDGET(m_resizer->get_widget()), 1);
+
+			gtk_box_reorder_child(m_search_box, GTK_WIDGET(m_search_entry), 0);
+			gtk_box_reorder_child(m_search_box, GTK_WIDGET(m_commands_align), 1);
+
+			gtk_box_reorder_child(m_contents_box, GTK_WIDGET(m_panels_box), 1);
+			gtk_box_reorder_child(m_contents_box, GTK_WIDGET(m_sidebar), 2);
+		}
+		else if (m_layout_commands_alternate)
+		{
+			gtk_misc_set_alignment(GTK_MISC(m_username), 1.0f, 0.5f);
+
+			gtk_alignment_set(m_commands_align, 0, 0, 0, 0);
+			gtk_box_reorder_child(m_commands_box, GTK_WIDGET(m_settings_button), 2);
+			gtk_box_reorder_child(m_commands_box, GTK_WIDGET(m_lock_screen_button), 1);
+			gtk_box_reorder_child(m_commands_box, GTK_WIDGET(m_log_out_button), 0);
+
+			gtk_box_reorder_child(m_title_box, GTK_WIDGET(m_username), 1);
+			gtk_box_reorder_child(m_title_box, GTK_WIDGET(m_resizer->get_widget()), 0);
+
+			gtk_box_reorder_child(m_search_box, GTK_WIDGET(m_search_entry), 1);
+			gtk_box_reorder_child(m_search_box, GTK_WIDGET(m_commands_align), 0);
+
+			gtk_box_reorder_child(m_contents_box, GTK_WIDGET(m_panels_box), 2);
+			gtk_box_reorder_child(m_contents_box, GTK_WIDGET(m_sidebar), 1);
+		}
+		else if (m_layout_left)
+		{
+			gtk_misc_set_alignment(GTK_MISC(m_username), 0.0f, 0.5f);
+
+			gtk_alignment_set(m_commands_align, 1, 0, 0, 0);
+			gtk_box_reorder_child(m_commands_box, GTK_WIDGET(m_settings_button), 0);
+			gtk_box_reorder_child(m_commands_box, GTK_WIDGET(m_lock_screen_button), 1);
+			gtk_box_reorder_child(m_commands_box, GTK_WIDGET(m_log_out_button), 2);
+
+			gtk_box_reorder_child(m_title_box, GTK_WIDGET(m_username), 0);
+			gtk_box_reorder_child(m_title_box, GTK_WIDGET(m_commands_align), 1);
+			gtk_box_reorder_child(m_title_box, GTK_WIDGET(m_resizer->get_widget()), 2);
 
 			gtk_box_reorder_child(m_contents_box, GTK_WIDGET(m_panels_box), 1);
 			gtk_box_reorder_child(m_contents_box, GTK_WIDGET(m_sidebar), 2);
@@ -389,10 +498,13 @@ void Menu::show(GtkWidget* parent, bool horizontal)
 		{
 			gtk_misc_set_alignment(GTK_MISC(m_username), 1.0f, 0.5f);
 
-			gtk_box_reorder_child(m_title_box, GTK_WIDGET(m_username), 4);
-			gtk_box_reorder_child(m_title_box, GTK_WIDGET(m_settings_button), 3);
-			gtk_box_reorder_child(m_title_box, GTK_WIDGET(m_lock_screen_button), 2);
-			gtk_box_reorder_child(m_title_box, GTK_WIDGET(m_log_out_button), 1);
+			gtk_alignment_set(m_commands_align, 0, 0, 0, 0);
+			gtk_box_reorder_child(m_commands_box, GTK_WIDGET(m_settings_button), 2);
+			gtk_box_reorder_child(m_commands_box, GTK_WIDGET(m_lock_screen_button), 1);
+			gtk_box_reorder_child(m_commands_box, GTK_WIDGET(m_log_out_button), 0);
+
+			gtk_box_reorder_child(m_title_box, GTK_WIDGET(m_username), 2);
+			gtk_box_reorder_child(m_title_box, GTK_WIDGET(m_commands_align), 1);
 			gtk_box_reorder_child(m_title_box, GTK_WIDGET(m_resizer->get_widget()), 0);
 
 			gtk_box_reorder_child(m_contents_box, GTK_WIDGET(m_panels_box), 2);
@@ -400,19 +512,32 @@ void Menu::show(GtkWidget* parent, bool horizontal)
 		}
 	}
 
-	if (layout_bottom != m_layout_bottom)
+	if ((layout_bottom != m_layout_bottom) || (m_layout_search_alternate != m_position_search_alternate))
 	{
 		m_layout_bottom = layout_bottom;
-		if (m_layout_bottom)
+		m_layout_search_alternate = m_position_search_alternate;
+		if (m_layout_bottom && m_layout_search_alternate)
 		{
 			gtk_box_reorder_child(m_vbox, GTK_WIDGET(m_title_box), 0);
-			gtk_box_reorder_child(m_vbox, GTK_WIDGET(m_search_entry), 1);
+			gtk_box_reorder_child(m_vbox, GTK_WIDGET(m_contents_box), 1);
+			gtk_box_reorder_child(m_vbox, GTK_WIDGET(m_search_box), 2);
+		}
+		else if (m_layout_search_alternate)
+		{
+			gtk_box_reorder_child(m_vbox, GTK_WIDGET(m_title_box), 2);
+			gtk_box_reorder_child(m_vbox, GTK_WIDGET(m_contents_box), 1);
+			gtk_box_reorder_child(m_vbox, GTK_WIDGET(m_search_box), 0);
+		}
+		else if (m_layout_bottom)
+		{
+			gtk_box_reorder_child(m_vbox, GTK_WIDGET(m_title_box), 0);
+			gtk_box_reorder_child(m_vbox, GTK_WIDGET(m_search_box), 1);
 			gtk_box_reorder_child(m_vbox, GTK_WIDGET(m_contents_box), 2);
 		}
 		else
 		{
 			gtk_box_reorder_child(m_vbox, GTK_WIDGET(m_title_box), 2);
-			gtk_box_reorder_child(m_vbox, GTK_WIDGET(m_search_entry), 1);
+			gtk_box_reorder_child(m_vbox, GTK_WIDGET(m_search_box), 1);
 			gtk_box_reorder_child(m_vbox, GTK_WIDGET(m_contents_box), 0);
 		}
 	}
@@ -448,7 +573,7 @@ void Menu::set_categories(const std::vector<SectionButton*>& categories)
 	}
 	gtk_widget_show_all(GTK_WIDGET(m_sidebar_box));
 
-	show_favorites();
+	show_default_page();
 }
 
 //-----------------------------------------------------------------------------
@@ -506,6 +631,94 @@ bool Menu::on_enter_notify_event(GdkEventCrossing* event)
 
 //-----------------------------------------------------------------------------
 
+std::string Menu::get_settings_command()
+{
+	return m_settings_command.m_exec;
+}
+
+//-----------------------------------------------------------------------------
+
+std::string Menu::get_lockscreen_command()
+{
+	return m_lockscreen_command.m_exec;
+}
+
+//-----------------------------------------------------------------------------
+
+std::string Menu::get_logout_command()
+{
+	return m_logout_command.m_exec;
+}
+
+//-----------------------------------------------------------------------------
+
+bool Menu::get_display_recent()
+{
+	return m_display_recent;
+}
+
+//-----------------------------------------------------------------------------
+
+bool Menu::get_position_search_alternate()
+{
+	return m_position_search_alternate;
+}
+
+//-----------------------------------------------------------------------------
+
+bool Menu::get_position_commands_alternate()
+{
+	return m_position_commands_alternate;
+}
+
+//-----------------------------------------------------------------------------
+
+void Menu::set_settings_command(const std::string& command)
+{
+	m_settings_command = command;
+}
+
+//-----------------------------------------------------------------------------
+
+void Menu::set_lockscreen_command(const std::string& command)
+{
+	m_lockscreen_command = command;
+}
+
+//-----------------------------------------------------------------------------
+
+void Menu::set_logout_command(const std::string& command)
+{
+	m_logout_command = command;
+}
+
+//-----------------------------------------------------------------------------
+
+void Menu::set_display_recent(bool display)
+{
+	m_display_recent = display;
+}
+
+//-----------------------------------------------------------------------------
+
+void Menu::set_position_search_alternate(bool alternate)
+{
+	m_position_search_alternate = alternate;
+	if (!alternate)
+	{
+		m_position_commands_alternate = false;
+	}
+}
+
+//-----------------------------------------------------------------------------
+
+void Menu::set_position_commands_alternate(bool alternate)
+{
+	m_position_commands_alternate = alternate && m_position_search_alternate;
+}
+
+//-----------------------------------------------------------------------------
+
 bool Menu::on_leave_notify_event(GdkEventCrossing* event)
 {
 	if ( (event->detail == GDK_NOTIFY_INFERIOR)
@@ -694,6 +907,18 @@ void Menu::show_favorites()
 
 //-----------------------------------------------------------------------------
 
+void Menu::show_default_page()
+{
+	// Switch to favorites panel
+	m_default_button->set_active(true);
+
+	// Clear search entry
+	gtk_entry_set_text(m_search_entry, "");
+	gtk_widget_grab_focus(GTK_WIDGET(m_search_entry));
+}
+
+//-----------------------------------------------------------------------------
+
 void Menu::search()
 {
 	// Fetch search string
@@ -738,7 +963,7 @@ void Menu::launch_settings_manager()
 	hide();
 
 	GError* error = NULL;
-	if (g_spawn_command_line_async("xfce4-settings-manager", &error) == false)
+	if (g_spawn_command_line_async(m_settings_command.m_exec.c_str(), &error) == false)
 	{
 		xfce_dialog_show_error(NULL, error, _("Failed to open settings manager."));
 		g_error_free(error);
@@ -752,7 +977,7 @@ void Menu::lock_screen()
 	hide();
 
 	GError* error = NULL;
-	if (g_spawn_command_line_async("xflock4", &error) == false)
+	if (g_spawn_command_line_async(m_lockscreen_command.m_exec.c_str(), &error) == false)
 	{
 		xfce_dialog_show_error(NULL, error, _("Failed to lock screen."));
 		g_error_free(error);
@@ -766,7 +991,7 @@ void Menu::log_out()
 	hide();
 
 	GError* error = NULL;
-	if (g_spawn_command_line_async("xfce4-session-logout", &error) == false)
+	if (g_spawn_command_line_async(m_logout_command.m_exec.c_str(), &error) == false)
 	{
 		xfce_dialog_show_error(NULL, error, _("Failed to log out."));
 		g_error_free(error);
@@ -774,3 +999,16 @@ void Menu::log_out()
 }
 
 //-----------------------------------------------------------------------------
+
+void Menu::check_command(Command& command, GtkWidget* button)
+{
+	if (command.m_status == Unchecked)
+	{
+		gchar* path = g_find_program_in_path(command.m_exec.c_str());
+		command.m_status = path ? Valid : Invalid;
+		g_free(path);
+	}
+	gtk_widget_set_sensitive(button, command.m_status == Valid);
+}
+
+//-----------------------------------------------------------------------------
